/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.jbi.management.task;

import java.io.File;

import org.ow2.petals.util.LoggingUtil;

import static org.ow2.petals.kernel.api.server.PetalsServer.INSTALLED_DIRECTORY;
import static org.ow2.petals.kernel.api.server.PetalsServer.WORK_DIRECTORY;

/**
 * Abstract Tack class for classes whose manipulate archive files.
 * 
 * @author ofabre
 * 
 */
public abstract class AbstractFileManipulationTask extends AbstractLoggableTask {

    private String petalsRootDirectory;

    public AbstractFileManipulationTask(LoggingUtil log, String petalsRootDirectory) {
        super(log);
        this.petalsRootDirectory = petalsRootDirectory;
    }

    /**
     * Get a file of the given name from the installed directory
     * 
     * @param file
     *            the file to look for
     * @return the {@ File}
     * 
     */
    public File getInstalledArchive(String archiveName) {
        return new File(this.petalsRootDirectory + File.separator + INSTALLED_DIRECTORY,
                archiveName);
    }

    /**
     * Retrieve work directory from AutoloaderServiceImpl
     * 
     * @return the work directory file
     */
    public File getWorkDirectory() {
        return new File(this.petalsRootDirectory + File.separator + WORK_DIRECTORY);
    }

}
