/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.installation.install.sl;

import java.net.URL;

import org.objectweb.fractal.api.Component;
import org.ow2.petals.container.ContainerService;
import org.ow2.petals.container.lifecycle.SharedLibraryLifeCycle;
import org.ow2.petals.jbi.descriptor.original.generated.ClassLoaderDelegationType;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi.SharedLibrary;
import org.ow2.petals.jbi.management.Context;
import org.ow2.petals.jbi.management.ManagementException;
import org.ow2.petals.jbi.management.admin.AdminService;
import org.ow2.petals.jbi.management.task.AbstractLoggableTask;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.kernel.server.FractalHelper;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task create the shared lib classloader <br/>
 * Inputs : {@link Jbi}, component install root URI <br/>
 * The task has got a non-robust approach : it stops at the first subtask failed<br />
 * and have the undo capability.<br />
 * 
 * @author ofabre
 * @author Frederic Gardes - eBM Websourcing
 */
public class CreateSLClassloaderTask extends AbstractLoggableTask {

    /**
     * JBI Admin Service :: JMX Agent Admin
     */
    private final AdminService adminService;

    /**
     * The Petals Container
     */
    protected ContainerService containerService;

    public CreateSLClassloaderTask(LoggingUtil log, AdminService adminService,
            ContainerService containerService) {
        super(log);
        this.containerService = containerService;
        this.adminService = adminService;
    }

    public void execute(Context context) throws Exception {
        this.log.start();

        Jbi descriptor = context.getDescriptor();
        URL installUrl = context.getInstallUrl();
        SharedLibrary sharedLibraryDescriptor = descriptor.getSharedLibrary();

        // Create shared lib classloader
        Component sharedLibraryLifeCycleComponent = this.containerService
                .createSharedLibraryLifeCycle(sharedLibraryDescriptor);

        // Start the new installer
        FractalHelper.startComponent(sharedLibraryLifeCycleComponent);
        SharedLibraryLifeCycle sharedLibraryLifeCycle = ((SharedLibraryLifeCycle) sharedLibraryLifeCycleComponent
                .getFcInterface("service"));

        boolean parentFirst = ClassLoaderDelegationType.PARENT_FIRST.equals(sharedLibraryDescriptor
                .getClassLoaderDelegation());
        sharedLibraryLifeCycle.init(sharedLibraryDescriptor, new URL[] { installUrl }, parentFirst);

        // load in PetalsShared Libraries
        sharedLibraryLifeCycle.loadSharedLibrary();
        this.adminService.registerSharedLibrary(sharedLibraryLifeCycle, sharedLibraryDescriptor
                .getIdentification().getName() + "-" + sharedLibraryDescriptor.getVersion());

        this.log.end();
    }

    public void undo(Context context) throws Exception {
        this.log.call();
        
        String slName = context.getEntityName();
        if (slName != null) {
            // Unregister
            try {
                this.adminService.unregisterSharedLibrary(slName);
            } catch (ManagementException e) {
                this.log.error("The shared library can't be unregistered or doesn't exist", e);
            }
            // Remove of the fractal installer component
            try {
                this.containerService.removeSharedLibraryLifeCycle(slName);
            } catch (PetalsException e) {
                this.log.error("The fractal shared library can't be removed or doesn't exist", e);
            }
        }
    }

}
