/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.installation.uninstall.sl;

import java.io.IOException;

import org.ow2.petals.jbi.management.Context;
import org.ow2.petals.jbi.management.task.AbstractLoggableTask;
import org.ow2.petals.system.repository.RepositoryService;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task remove the shared lib folder from the Petals repository <br/>
 * Inputs : shared lib name<br />
 * The task has got a robust approach : it tries to do all the subtasks<br />
 * and doesn't have the undo capability.<br />
 * 
 * @author ofabre - EBM Websourcing
 * @author fgardes - EBM Websourcing
 */
public class RemoveSLRepositoryTask extends AbstractLoggableTask {

    private final RepositoryService repositoryService;

    public RemoveSLRepositoryTask(LoggingUtil log, RepositoryService repositoryService) {
        super(log);
        this.repositoryService = repositoryService;
    }

    public void execute(Context context) {
        String slName = context.getEntityName();
        String version = context.getEntityVersion();
        try {
            this.repositoryService.removeSharedLibrary(slName + "-" + version);
        } catch (IOException e) {
            this.log.error("The shared library '" + slName + "' can not be removed from repository",
                    e);
        }
    }

    public void undo(Context context) throws Exception {
        // TODO : recreate the shared library folder
    }

}
